@@ -6,13 +6,15 @@ import (
 	"regexp"
 
 	"github.com/go-logr/logr"
+	ocpconfigv1 "github.com/openshift/api/config/v1"
 	routev1 "github.com/openshift/api/route/v1"
 	v1 "k8s.io/api/core/v1"
-	extensionsv1beta1 "k8s.io/api/extensions/v1beta1"
+	networkingv1 "k8s.io/api/networking/v1"
 	storagev1 "k8s.io/api/storage/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/api/meta"
 	"k8s.io/apimachinery/pkg/api/resource"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/types"
 	"sigs.k8s.io/controller-runtime/pkg/cache"
@@ -25,7 +27,7 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 	"sigs.k8s.io/controller-runtime/pkg/source"
 
-	cdiv1 "kubevirt.io/containerized-data-importer/pkg/apis/core/v1beta1"
+	cdiv1 "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1"
 	"kubevirt.io/containerized-data-importer/pkg/common"
 	"kubevirt.io/containerized-data-importer/pkg/operator"
 	"kubevirt.io/containerized-data-importer/pkg/util"
@@ -34,6 +36,10 @@ import (
 // AnnConfigAuthority is the annotation specifying a resource as the CDIConfig authority
 const (
 	AnnConfigAuthority = "cdi.kubevirt.io/configAuthority"
+	defaultCPULimit    = "750m"
+	defaultMemLimit    = "600M"
+	defaultCPURequest  = "100m"
+	defaultMemRequest  = "60M"
 )
 
 // CDIConfigReconciler members
@@ -46,17 +52,19 @@ type CDIConfigReconciler struct {
 	uploadProxyServiceName string
 	configName             string
 	cdiNamespace           string
+	installerLabels        map[string]string
 }
 
 func isErrCacheNotStarted(err error) bool {
 	if err == nil {
 		return false
 	}
-	return err.(*cache.ErrCacheNotStarted) != nil
+	_, ok := err.(*cache.ErrCacheNotStarted)
+	return ok
 }
 
 // Reconcile the reconcile loop for the CDIConfig object.
-func (r *CDIConfigReconciler) Reconcile(req reconcile.Request) (reconcile.Result, error) {
+func (r *CDIConfigReconciler) Reconcile(_ context.Context, req reconcile.Request) (reconcile.Result, error) {
 	log := r.log.WithValues("CDIConfig", req.NamespacedName)
 	log.Info("reconciling CDIConfig")
 
@@ -75,7 +83,7 @@ func (r *CDIConfigReconciler) Reconcile(req reconcile.Request) (reconcile.Result
 		return reconcile.Result{}, err
 	}
 
-	if err := r.reconcileUploadProxyURL(config, log); err != nil {
+	if err := r.reconcileUploadProxyURL(config); err != nil {
 		return reconcile.Result{}, err
 	}
 
@@ -91,6 +99,10 @@ func (r *CDIConfigReconciler) Reconcile(req reconcile.Request) (reconcile.Result
 		return reconcile.Result{}, err
 	}
 
+	if err := r.reconcileImportProxy(config); err != nil {
+		return reconcile.Result{}, err
+	}
+
 	if !reflect.DeepEqual(currentConfigCopy, config) {
 		// Updates have happened, update CDIConfig.
 		log.Info("Updating CDIConfig", "CDIConfig.Name", config.Name, "config", config)
@@ -103,6 +115,8 @@ func (r *CDIConfigReconciler) Reconcile(req reconcile.Request) (reconcile.Result
 }
 
 func (r *CDIConfigReconciler) setOperatorParams(config *cdiv1.CDIConfig) error {
+	util.SetRecommendedLabels(config, r.installerLabels, "cdi-controller")
+
 	cdiCR, err := GetActiveCDI(r.client)
 	if err != nil {
 		return err
@@ -125,7 +139,8 @@ func (r *CDIConfigReconciler) setOperatorParams(config *cdiv1.CDIConfig) error {
 	return nil
 }
 
-func (r *CDIConfigReconciler) reconcileUploadProxyURL(config *cdiv1.CDIConfig, log logr.Logger) error {
+func (r *CDIConfigReconciler) reconcileUploadProxyURL(config *cdiv1.CDIConfig) error {
+	log := r.log.WithName("CDIconfig").WithName("UploadProxyReconcile")
 	config.Status.UploadProxyURL = config.Spec.UploadProxyURLOverride
 	// No override, try Ingress
 	if config.Status.UploadProxyURL == nil {
@@ -146,7 +161,7 @@ func (r *CDIConfigReconciler) reconcileUploadProxyURL(config *cdiv1.CDIConfig, l
 
 func (r *CDIConfigReconciler) reconcileIngress(config *cdiv1.CDIConfig) error {
 	log := r.log.WithName("CDIconfig").WithName("IngressReconcile")
-	ingressList := &extensionsv1beta1.IngressList{}
+	ingressList := &networkingv1.IngressList{}
 	if err := r.client.List(context.TODO(), ingressList, &client.ListOptions{}); IgnoreIsNoMatchError(err) != nil {
 		return err
 	}
@@ -216,13 +231,19 @@ func (r *CDIConfigReconciler) reconcileStorageClass(config *cdiv1.CDIConfig) err
 }
 
 func (r *CDIConfigReconciler) reconcileDefaultPodResourceRequirements(config *cdiv1.CDIConfig) error {
+	cpuLimit, _ := resource.ParseQuantity(defaultCPULimit)
+	memLimit, _ := resource.ParseQuantity(defaultMemLimit)
+	cpuRequest, _ := resource.ParseQuantity(defaultCPURequest)
+	memRequest, _ := resource.ParseQuantity(defaultMemRequest)
 	config.Status.DefaultPodResourceRequirements = &v1.ResourceRequirements{
 		Limits: map[v1.ResourceName]resource.Quantity{
-			v1.ResourceCPU:    *resource.NewQuantity(0, resource.DecimalSI),
-			v1.ResourceMemory: *resource.NewQuantity(0, resource.DecimalSI)},
+			v1.ResourceCPU:    cpuLimit,
+			v1.ResourceMemory: memLimit,
+		},
 		Requests: map[v1.ResourceName]resource.Quantity{
-			v1.ResourceCPU:    *resource.NewQuantity(0, resource.DecimalSI),
-			v1.ResourceMemory: *resource.NewQuantity(0, resource.DecimalSI)},
+			v1.ResourceCPU:    cpuRequest,
+			v1.ResourceMemory: memRequest,
+		},
 	}
 
 	if config.Spec.PodResourceRequirements != nil {
@@ -316,6 +337,7 @@ func (r *CDIConfigReconciler) createCDIConfig() (*cdiv1.CDIConfig, error) {
 			if err := operator.SetOwnerRuntime(r.uncachedClient, config); err != nil {
 				return nil, err
 			}
+			util.SetRecommendedLabels(config, r.installerLabels, "cdi-controller")
 			if err := r.client.Create(context.TODO(), config); err != nil {
 				if errors.IsAlreadyExists(err) {
 					config := &cdiv1.CDIConfig{}
@@ -333,6 +355,79 @@ func (r *CDIConfigReconciler) createCDIConfig() (*cdiv1.CDIConfig, error) {
 	return config, nil
 }
 
+func (r *CDIConfigReconciler) reconcileImportProxy(config *cdiv1.CDIConfig) error {
+	config.Status.ImportProxy = config.Spec.ImportProxy
+
+	// Avoid nil pointers and segfaults for the initial case, where ImportProxy is nil for both the spec and the status.
+	if config.Status.ImportProxy == nil {
+		config.Status.ImportProxy = &cdiv1.ImportProxy{
+			HTTPProxy:      new(string),
+			HTTPSProxy:     new(string),
+			NoProxy:        new(string),
+			TrustedCAProxy: new(string),
+		}
+
+		// Try Openshift cluster wide proxy only if the CDIConfig default config is empty
+		clusterWideProxy, err := GetClusterWideProxy(r.client)
+		if err != nil {
+			return err
+		}
+		config.Status.ImportProxy.HTTPProxy = &clusterWideProxy.Status.HTTPProxy
+		config.Status.ImportProxy.HTTPSProxy = &clusterWideProxy.Status.HTTPSProxy
+		config.Status.ImportProxy.NoProxy = &clusterWideProxy.Status.NoProxy
+		if clusterWideProxy.Spec.TrustedCA.Name != "" {
+			config.Status.ImportProxy.TrustedCAProxy = &clusterWideProxy.Spec.TrustedCA.Name
+			err = r.reconcileImportProxyCAConfigMap(config, clusterWideProxy)
+			if err != nil {
+				return err
+			}
+		}
+	}
+	return nil
+}
+
+// Create/Update a configmap with the CA certificates in the controllor context with the cluster-wide proxy CA certificates to be used by the importer pod
+func (r *CDIConfigReconciler) reconcileImportProxyCAConfigMap(config *cdiv1.CDIConfig, proxy *ocpconfigv1.Proxy) error {
+	client := r.uncachedClient
+	cmName := proxy.Spec.TrustedCA.Name
+	if cmName == "" {
+		// Using the default cluster-wide proxy CA certificates configmap name
+		cmName = ClusterWideProxyConfigMapName
+	}
+	clusterWideProxyConfigMap := &v1.ConfigMap{}
+	if err := client.Get(context.TODO(), types.NamespacedName{Name: cmName, Namespace: ClusterWideProxyConfigMapNameSpace}, clusterWideProxyConfigMap); err == nil {
+		// Copy the cluster-wide proxy CA certificates to the importer pod proxy CA certificates configMap
+		if certBytes, ok := clusterWideProxyConfigMap.Data[ClusterWideProxyConfigMapKey]; ok {
+			configMap := &v1.ConfigMap{}
+			if err := client.Get(context.TODO(), types.NamespacedName{Name: common.ImportProxyConfigMapName, Namespace: r.cdiNamespace}, configMap); errors.IsNotFound(err) {
+				proxyConfigMap := r.createProxyConfigMap(certBytes)
+				util.SetRecommendedLabels(proxyConfigMap, r.installerLabels, "cdi-controller")
+				if err := client.Create(context.TODO(), proxyConfigMap); err != nil {
+					return err
+				}
+				return nil
+			}
+			if configMap != nil {
+				configMap.Data[common.ImportProxyConfigMapKey] = certBytes
+				util.SetRecommendedLabels(configMap, r.installerLabels, "cdi-controller")
+				if err := client.Update(context.TODO(), configMap); err != nil {
+					return err
+				}
+			}
+		}
+	}
+	return nil
+}
+
+func (r *CDIConfigReconciler) createProxyConfigMap(certBytes string) *v1.ConfigMap {
+	return &v1.ConfigMap{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      common.ImportProxyConfigMapName,
+			Namespace: r.cdiNamespace},
+		Data: map[string]string{common.ImportProxyConfigMapKey: certBytes},
+	}
+}
+
 // Init initializes a CDIConfig object.
 func (r *CDIConfigReconciler) Init() error {
 	_, err := r.createCDIConfig()
@@ -340,7 +435,7 @@ func (r *CDIConfigReconciler) Init() error {
 }
 
 // NewConfigController creates a new instance of the config controller.
-func NewConfigController(mgr manager.Manager, log logr.Logger, uploadProxyServiceName, configName string) (controller.Controller, error) {
+func NewConfigController(mgr manager.Manager, log logr.Logger, uploadProxyServiceName, configName string, installerLabels map[string]string) (controller.Controller, error) {
 	uncachedClient, err := client.New(mgr.GetConfig(), client.Options{
 		Scheme: mgr.GetScheme(),
 		Mapper: mgr.GetRESTMapper(),
@@ -356,6 +451,7 @@ func NewConfigController(mgr manager.Manager, log logr.Logger, uploadProxyServic
 		uploadProxyServiceName: uploadProxyServiceName,
 		configName:             configName,
 		cdiNamespace:           util.GetNamespace(),
+		installerLabels:        installerLabels,
 	}
 
 	configController, err := controller.New("config-controller", mgr, controller.Options{
@@ -364,7 +460,7 @@ func NewConfigController(mgr manager.Manager, log logr.Logger, uploadProxyServic
 	if err != nil {
 		return nil, err
 	}
-	if err := addConfigControllerWatches(mgr, configController, reconciler.cdiNamespace, configName, uploadProxyServiceName); err != nil {
+	if err := addConfigControllerWatches(mgr, configController, reconciler.cdiNamespace, configName, uploadProxyServiceName, log); err != nil {
 		return nil, err
 	}
 	if err := reconciler.Init(); err != nil {
@@ -375,7 +471,7 @@ func NewConfigController(mgr manager.Manager, log logr.Logger, uploadProxyServic
 }
 
 // addConfigControllerWatches sets up the watches used by the config controller.
-func addConfigControllerWatches(mgr manager.Manager, configController controller.Controller, cdiNamespace, configName, uploadProxyServiceName string) error {
+func addConfigControllerWatches(mgr manager.Manager, configController controller.Controller, cdiNamespace, configName, uploadProxyServiceName string, log logr.Logger) error {
 	// Add schemes.
 	if err := cdiv1.AddToScheme(mgr.GetScheme()); err != nil {
 		return err
@@ -383,95 +479,136 @@ func addConfigControllerWatches(mgr manager.Manager, configController controller
 	if err := storagev1.AddToScheme(mgr.GetScheme()); err != nil {
 		return err
 	}
-	if err := extensionsv1beta1.AddToScheme(mgr.GetScheme()); err != nil {
+	if err := networkingv1.AddToScheme(mgr.GetScheme()); err != nil {
 		return err
 	}
-	if err := routev1.AddToScheme(mgr.GetScheme()); err != nil {
+	if err := routev1.Install(mgr.GetScheme()); err != nil {
+		return err
+	}
+	if err := ocpconfigv1.Install(mgr.GetScheme()); err != nil {
 		return err
 	}
 
 	// Setup watches
-	if err := configController.Watch(&source.Kind{Type: &cdiv1.CDIConfig{}}, &handler.EnqueueRequestForObject{}); err != nil {
+	if err := watchCDIConfig(configController, configName); err != nil {
 		return err
 	}
-	err := configController.Watch(&source.Kind{Type: &cdiv1.CDI{}}, &handler.EnqueueRequestsFromMapFunc{
-		ToRequests: handler.ToRequestsFunc(func(obj handler.MapObject) []reconcile.Request {
-			return []reconcile.Request{{
-				NamespacedName: types.NamespacedName{Name: configName},
-			}}
-		}),
-	})
-	if err != nil {
+	if err := watchStorageClass(configController, configName); err != nil {
 		return err
 	}
-	err = configController.Watch(&source.Kind{Type: &storagev1.StorageClass{}}, &handler.EnqueueRequestsFromMapFunc{
-		ToRequests: handler.ToRequestsFunc(func(obj handler.MapObject) []reconcile.Request {
-			return []reconcile.Request{{
-				NamespacedName: types.NamespacedName{Name: configName},
-			}}
-		}),
-	})
-	if err != nil {
+	if err := watchIngress(configController, cdiNamespace, configName, uploadProxyServiceName); err != nil {
 		return err
 	}
-	err = configController.Watch(&source.Kind{Type: &extensionsv1beta1.Ingress{}}, &handler.EnqueueRequestsFromMapFunc{
-		ToRequests: handler.ToRequestsFunc(func(obj handler.MapObject) []reconcile.Request {
-			return []reconcile.Request{{
-				NamespacedName: types.NamespacedName{Name: configName},
-			}}
-		}),
-	}, predicate.Funcs{
-		CreateFunc: func(e event.CreateEvent) bool {
-			return "" != getURLFromIngress(e.Object.(*extensionsv1beta1.Ingress), uploadProxyServiceName) &&
-				e.Object.(*extensionsv1beta1.Ingress).GetNamespace() == cdiNamespace
-		},
-		UpdateFunc: func(e event.UpdateEvent) bool {
-			return "" != getURLFromIngress(e.ObjectNew.(*extensionsv1beta1.Ingress), uploadProxyServiceName) &&
-				e.ObjectNew.(*extensionsv1beta1.Ingress).GetNamespace() == cdiNamespace
-		},
-		DeleteFunc: func(e event.DeleteEvent) bool {
-			return "" != getURLFromIngress(e.Object.(*extensionsv1beta1.Ingress), uploadProxyServiceName) &&
-				e.Object.(*extensionsv1beta1.Ingress).GetNamespace() == cdiNamespace
-		},
-	})
-
-	// check if routes exist
-	err = mgr.GetClient().List(context.TODO(), &routev1.RouteList{})
-	if meta.IsNoMatchError(err) {
-		return nil
+	if err := watchRoutes(mgr, configController, cdiNamespace, configName, uploadProxyServiceName); err != nil {
+		return err
 	}
-
-	if err != nil && !isErrCacheNotStarted(err) {
+	if err := watchClusterProxy(mgr, configController, configName); err != nil {
 		return err
 	}
-
-	err = configController.Watch(&source.Kind{Type: &routev1.Route{}}, &handler.EnqueueRequestsFromMapFunc{
-		ToRequests: handler.ToRequestsFunc(func(obj handler.MapObject) []reconcile.Request {
-			return []reconcile.Request{{
-				NamespacedName: types.NamespacedName{Name: configName},
-			}}
-		}),
-	}, predicate.Funcs{
-		CreateFunc: func(e event.CreateEvent) bool {
-			return "" != getURLFromRoute(e.Object.(*routev1.Route), uploadProxyServiceName) &&
-				e.Object.(*routev1.Route).GetNamespace() == cdiNamespace
-		},
-		UpdateFunc: func(e event.UpdateEvent) bool {
-			return "" != getURLFromRoute(e.ObjectNew.(*routev1.Route), uploadProxyServiceName) &&
-				e.ObjectNew.(*routev1.Route).GetNamespace() == cdiNamespace
-		},
-		DeleteFunc: func(e event.DeleteEvent) bool {
-			return "" != getURLFromRoute(e.Object.(*routev1.Route), uploadProxyServiceName) &&
-				e.Object.(*routev1.Route).GetNamespace() == cdiNamespace
-		},
-	})
 
 	return nil
 }
 
-func getURLFromIngress(ing *extensionsv1beta1.Ingress, uploadProxyServiceName string) string {
-	if ing.Spec.Backend != nil {
-		if ing.Spec.Backend.ServiceName != uploadProxyServiceName {
+func watchCDIConfig(configController controller.Controller, configName string) error {
+	if err := configController.Watch(&source.Kind{Type: &cdiv1.CDIConfig{}}, &handler.EnqueueRequestForObject{}); err != nil {
+		return err
+	}
+	return configController.Watch(&source.Kind{Type: &cdiv1.CDI{}}, handler.EnqueueRequestsFromMapFunc(
+		func(client.Object) []reconcile.Request {
+			return []reconcile.Request{{
+				NamespacedName: types.NamespacedName{Name: configName},
+			}}
+		},
+	))
+}
+
+func watchStorageClass(configController controller.Controller, configName string) error {
+	return configController.Watch(&source.Kind{Type: &storagev1.StorageClass{}}, handler.EnqueueRequestsFromMapFunc(
+		func(client.Object) []reconcile.Request {
+			return []reconcile.Request{{
+				NamespacedName: types.NamespacedName{Name: configName},
+			}}
+		},
+	))
+}
+
+func watchIngress(configController controller.Controller, cdiNamespace, configName, uploadProxyServiceName string) error {
+	err := configController.Watch(&source.Kind{Type: &networkingv1.Ingress{}}, handler.EnqueueRequestsFromMapFunc(
+		func(client.Object) []reconcile.Request {
+			return []reconcile.Request{{
+				NamespacedName: types.NamespacedName{Name: configName},
+			}}
+		}),
+		predicate.Funcs{
+			CreateFunc: func(e event.CreateEvent) bool {
+				return "" != getURLFromIngress(e.Object.(*networkingv1.Ingress), uploadProxyServiceName) &&
+					e.Object.(*networkingv1.Ingress).GetNamespace() == cdiNamespace
+			},
+			UpdateFunc: func(e event.UpdateEvent) bool {
+				return "" != getURLFromIngress(e.ObjectNew.(*networkingv1.Ingress), uploadProxyServiceName) &&
+					e.ObjectNew.(*networkingv1.Ingress).GetNamespace() == cdiNamespace
+			},
+			DeleteFunc: func(e event.DeleteEvent) bool {
+				return "" != getURLFromIngress(e.Object.(*networkingv1.Ingress), uploadProxyServiceName) &&
+					e.Object.(*networkingv1.Ingress).GetNamespace() == cdiNamespace
+			},
+		})
+	return err
+}
+
+// we only watch the route obj if they exist, i.e., if it is an OpenShift cluster
+func watchRoutes(mgr manager.Manager, configController controller.Controller, cdiNamespace, configName, uploadProxyServiceName string) error {
+	err := mgr.GetClient().List(context.TODO(), &routev1.RouteList{})
+	if !meta.IsNoMatchError(err) {
+		if err == nil || isErrCacheNotStarted(err) {
+			err := configController.Watch(&source.Kind{Type: &routev1.Route{}}, handler.EnqueueRequestsFromMapFunc(
+				func(client.Object) []reconcile.Request {
+					return []reconcile.Request{{
+						NamespacedName: types.NamespacedName{Name: configName},
+					}}
+				}),
+				predicate.Funcs{
+					CreateFunc: func(e event.CreateEvent) bool {
+						return "" != getURLFromRoute(e.Object.(*routev1.Route), uploadProxyServiceName) &&
+							e.Object.(*routev1.Route).GetNamespace() == cdiNamespace
+					},
+					UpdateFunc: func(e event.UpdateEvent) bool {
+						return "" != getURLFromRoute(e.ObjectNew.(*routev1.Route), uploadProxyServiceName) &&
+							e.ObjectNew.(*routev1.Route).GetNamespace() == cdiNamespace
+					},
+					DeleteFunc: func(e event.DeleteEvent) bool {
+						return "" != getURLFromRoute(e.Object.(*routev1.Route), uploadProxyServiceName) &&
+							e.Object.(*routev1.Route).GetNamespace() == cdiNamespace
+					},
+				})
+			return err
+		}
+		return err
+	}
+	return nil
+}
+
+// we only watch the cluster-wide proxy obj if they exist, i.e., if it is an OpenShift cluster
+func watchClusterProxy(mgr manager.Manager, configController controller.Controller, configName string) error {
+	err := mgr.GetClient().List(context.TODO(), &ocpconfigv1.ProxyList{})
+	if !meta.IsNoMatchError(err) {
+		if err == nil || isErrCacheNotStarted(err) {
+			return configController.Watch(&source.Kind{Type: &ocpconfigv1.Proxy{}}, handler.EnqueueRequestsFromMapFunc(
+				func(client.Object) []reconcile.Request {
+					return []reconcile.Request{{
+						NamespacedName: types.NamespacedName{Name: configName},
+					}}
+				},
+			))
+		}
+		return err
+	}
+	return nil
+}
+
+func getURLFromIngress(ing *networkingv1.Ingress, uploadProxyServiceName string) string {
+	if ing.Spec.DefaultBackend != nil && ing.Spec.DefaultBackend.Service != nil {
+		if ing.Spec.DefaultBackend.Service.Name != uploadProxyServiceName {
 			return ""
 		}
 		return ing.Spec.Rules[0].Host
@@ -481,7 +618,7 @@ func getURLFromIngress(ing *extensionsv1beta1.Ingress, uploadProxyServiceName st
 			continue
 		}
 		for _, path := range rule.HTTP.Paths {
-			if path.Backend.ServiceName == uploadProxyServiceName {
+			if path.Backend.Service != nil && path.Backend.Service.Name == uploadProxyServiceName {
 				if rule.Host != "" {
 					return rule.Host
 				}
@@ -489,7 +626,6 @@ func getURLFromIngress(ing *extensionsv1beta1.Ingress, uploadProxyServiceName st
 		}
 	}
 	return ""
-
 }
 
 func getURLFromRoute(route *routev1.Route, uploadProxyServiceName string) string {
